

///////////////////////////////////////////////////////////////////////////////
// This C example is meant as a training vehicle for our C API.  This will initialize the Speech Port,
// load up some sample grammar concepts and a SRGS grammar, then open up an audio file for
// streaming to our Engine for decoding, and display the results.  The raw audio file is in 8k u-law. 
// This example will detect the end of speech then get the results from the Engine.
///////////////////////////////////////////////////////////////////////////////

#include <LV_SRE.h>
#include <windows.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>
#include <conio.h>


// Handle to the speech port we will use
HPORT hPort;

// Using voice channel 1
int VoiceChannel = 1;

// Flag to indicate end of decode
bool SpeechDecoded = false;


// The call back function will check the various states of the engine.  For this example we only care about
// the STREAM_STATUS_STOPPED.  When this event occurs then we will wait for the engine to idle which tells
// us that the decode has been performed.  When the decode is done then we get the results and display them.
//
void port_callback(int StreamStatus, unsigned int TotalBytes, unsigned int RecordedBytes, void *UserData)
{

	int iDecodeStatus;
	int iIndex = 0;

	printf("Port Callback: ");

	switch(StreamStatus)
	{
	case STREAM_STATUS_NOT_READY:
		printf("Not Ready\n");
		break;

	case STREAM_STATUS_READY:
		printf("Ready\n");
		break;

	case STREAM_STATUS_BARGE_IN:
		printf("Barge In\n");
		break;

	case STREAM_STATUS_END_SPEECH:
		printf("End of Speech\n");

		iDecodeStatus = LV_SRE_WaitForEngineToIdle(hPort, VoiceChannel, 7000);

		if(iDecodeStatus != LV_TIME_OUT)
		{
			printf("\nDecoded Transcript:\t%s\nConfidence Score:\t%d\nTotal Number of Interpretations:\t%d\n\n",

				LV_SRE_GetInterpretationString(hPort, VoiceChannel, iIndex),
				LV_SRE_GetInterpretationScore(hPort, VoiceChannel, iIndex),
				LV_SRE_GetNumberOfInterpretations(hPort, VoiceChannel));
		}

		// Set a flag to let the audio streamer know that the decode is complete
		SpeechDecoded = true;
		break;

	case STREAM_STATUS_STOPPED:
		printf("Stream Stopped\n");
		break;

	case STREAM_STATUS_BARGE_IN_TIMEOUT:
		printf("Barge In Time Out\n");
		break;

	case STREAM_STATUS_END_SPEECH_TIMEOUT:
		printf("EOS Time Out\n");
		break;

	default:
		printf("Unexpected Stream Status (%d)\n", StreamStatus);
		break;
	}
}


// This function will open the audio file for streaming, fill up the stream buffers, and
// send the buffers to the engine
//
int Audio_Stream(char * AudioFilename)
{
	int ErrorCode;
	int AudioBufferSize = 0;
	int IncrementMs = 300;
	int BytesPerSample = 1;
	int SamplesPerSecond = 8000;
	int ChunkSize;
	char * AudioBuffer;
	char* CurrentPos;
	int AudioLength=0;
	int AudioHandle;
	FILE *fpAudio;


	// Attempt to open the audio file
	AudioHandle = _open(AudioFilename, _O_BINARY | _O_RDONLY);

	if(AudioHandle < 0)
	{
		printf("Error loading audio file.\n");
		return LV_FAILURE;
	}

	// Figure out the length of the file
	AudioBufferSize = _lseek(AudioHandle, 0L,SEEK_END);

	// Allocate a memory buffer big enough to hold the entire file
	AudioBuffer = (char *)malloc(AudioBufferSize);

	// Now that we know the length of the file, rewind to the beginning
	_lseek(AudioHandle, 0L, SEEK_SET);

	// Read the audio file into the audio buffer
	_read(AudioHandle, AudioBuffer, AudioBufferSize);

	// Close the audio file (we don't need it any more)
	_close(AudioHandle);


	// Point to the beginning of the AudioBuffer
	CurrentPos = AudioBuffer;


	// Calculate chunk size - For example (1 byte per sample * 8000 samples per sec * 300mS / 1000) = 2400 bytes
	ChunkSize = BytesPerSample * SamplesPerSecond * IncrementMs / 1000;
	

	// Create a BlankBuffer to add silence to end of speech
	char *BlankBuffer = (char *)malloc(ChunkSize);

	memset(BlankBuffer, 0, ChunkSize);


	if((ErrorCode = LV_SRE_StreamStart(hPort)) != LV_SUCCESS)
	{
		printf("Error Starting Stream: %s\n", LV_SRE_ReturnErrorString(ErrorCode));

		// Clean up and return...
		free(BlankBuffer);
		free(AudioBuffer);
		return ErrorCode;
	}
	

	printf("Streaming the audio.\n");



	// Stream the buffer at 300mS intervals to the SRE until all sent
	while(CurrentPos != AudioBuffer + AudioBufferSize)
	{
		if(AudioLength + ChunkSize > AudioBufferSize)
		{
			ChunkSize = AudioBufferSize - AudioLength;
		}

		AudioLength +=ChunkSize;

		// Sending the filled buffers to our Engine
		ErrorCode = LV_SRE_StreamSendData(hPort,CurrentPos,ChunkSize);

		if(ErrorCode != LV_SUCCESS)
			printf("Error sending stream data: %s\n", LV_SRE_ReturnErrorString(ErrorCode));


		CurrentPos += ChunkSize;

		Sleep(100); // Adding artificial delay here to simulate live streaming 
	}


	// Add 900mS of silence to the end of the stream to allow the streamer to correctly
	// detect end of speech, simulating a normal streaming condition.
	//for(int FillerCount = 0; FillerCount < 1; FillerCount++)
	//{
	//	// Sending empty buffers to the Speech Engine
	//	ErrorCode = LV_SRE_StreamSendData(hPort, BlankBuffer, ChunkSize);
	//	if(ErrorCode != LV_SUCCESS)
	//		printf("Error sending 'blank' stream data: %s\n", LV_SRE_ReturnErrorString(ErrorCode));
	//}


	int WaitCount;

	// Wait for up to 2 seconds for speech decode to finish
	for(WaitCount = 0; WaitCount < 200; WaitCount++)
	{
		if(SpeechDecoded == true)
			break;
		Sleep(10);
	}

	if(SpeechDecoded)
		printf("Waited %d mS for decode to finish\n", WaitCount * 10);
	else
		printf("Speech not decoded in time\n");

	// Release the memory allocated to the audio buffers
	free(BlankBuffer);
	free(AudioBuffer);

	printf("\nCompleted audio streaming.\n");

	return LV_SUCCESS;
}


// Load and activate the specified grammar
//
int LoadGrammar(char * GrammarLabel, char * GrammarFile)
{
	int ErrorCode;

	printf("Loading the grammar and activating it.\n");

	// Here we are loading up a SRGS grammar.
	ErrorCode = LV_SRE_LoadGrammar(hPort, GrammarLabel, GrammarFile);

	if(ErrorCode != LV_SUCCESS)
	{
		printf("Error Loading Grammar: %s\n", LV_SRE_ReturnErrorString(ErrorCode));
		return ErrorCode;
	}

	// Once you load up a grammar you can activate and deactivate it at any point during the call flow.
	// For this example we will activate it immediately.
	ErrorCode = LV_SRE_ActivateGrammar(hPort, GrammarLabel);

	if(ErrorCode != LV_SUCCESS)
	{
		printf("Error Activating Grammar: %s\n", LV_SRE_ReturnErrorString(ErrorCode));
		return ErrorCode;
	}

	return LV_SUCCESS;
}


// This helper routine calls LV_SRE_StreamSetParameter and checks the results, displaying
// the error string if problems are detected
int SetStreamParameterHelper(HPORT hPort, int ParameterType, unsigned long Value)
{
	int ErrorCode = LV_SRE_StreamSetParameter(hPort, ParameterType, Value);

	if(ErrorCode != LV_SUCCESS)
		printf("Parameter Error: %s\n", LV_SRE_ReturnErrorString(ErrorCode));

	return ErrorCode;
}


// Initialize the speech port with streaming parameters
//
int InitializePort()
{
	int ErrorCode;

	// Open the speech port
	//
	hPort = LV_SRE_CreateClient(&ErrorCode, NULL, NULL,0);

	if(ErrorCode != LV_SUCCESS)
	{
		printf("Problem Initializing the port: %s\n", LV_SRE_ReturnErrorString(ErrorCode));
		return ErrorCode;
	}

	printf("Initializing the port.\n");

	// Use the next line to configure for the SRE on a machine other than the local (uses local machine without this)
	ErrorCode = LV_SRE_SetPropertyEx(hPort, PROP_EX_SRE_SERVERS, PROP_EX_VALUE_TYPE_STRING, "127.0.0.1:5730", PROP_EX_TARGET_CLIENT, VoiceChannel);

	if(ErrorCode != LV_SUCCESS)
	{
		printf("Problem selecting server: %s\n", LV_SRE_ReturnErrorString(ErrorCode));
		return ErrorCode;
	}

	// Log the results to a file
	ErrorCode = LV_SRE_SetPropertyEx(hPort, PROP_EX_SAVE_SOUND_FILES, PROP_EX_VALUE_TYPE_INT, (void*)1, PROP_EX_TARGET_CLIENT, VoiceChannel);

	// VAD Related
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_END_OF_SPEECH_TIMEOUT, 20000);
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_BARGE_IN_TIMEOUT, 15000);
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_VAD_EOS_DELAY, 350); // was 500
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_DETECT_BARGE_IN, 1);
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_DETECT_END_OF_SPEECH, 1);
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_VAD_WIND_BACK,250);	// was 1000
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_VAD_BURST_THLD,100);	// was 30
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_VAD_VOLUME_SENSITIVITY, 50);
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_VAD_SNR_SENSITIVITY, 50);
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_VAD_INIT_MODE, STREAM_PARM_VAD_INIT_SILENCE_TRIMMED);

	//Engine Related
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_VOICE_CHANNEL, VoiceChannel);
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_GRAMMAR_SET, LV_ACTIVE_GRAMMAR_SET);
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_AUTO_DECODE, 1);
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_DECODE_FLAGS, LV_DECODE_SEMANTIC_INTERPRETATION);
	ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_SOUND_FORMAT, ULAW_8KHZ);

	//ErrorCode = SetStreamParameterHelper(hPort, STREAM_PARM_SOUND_FORMAT, ULAW_8KHZ);

	LV_SRE_StreamSetStateChangeCallBack(hPort, port_callback, NULL);

	return LV_SUCCESS;
}


// This is the main application routine that will initialize the speech system,
// load and activate the grammar file then stream a specified audio file to the
// speech engine that should be automatically decoded at the end of the audio
//
void main()
{

	// Open and initialize the speech port
	if(InitializePort() == LV_SUCCESS)
	{

		char *grammar_file_path = "ABNFDigits.gram";	// This is the grammar file we'll use
		char *audio_file_path = "8587070707.ulaw";		// This is the audio file we'll use for the decode


		// Load and activate the grammar file and assign it a label  of "GrammarLabel" so
		// that we can refer to it later if needed.
		//
		if(LoadGrammar("GrammarLabel", grammar_file_path) == LV_SUCCESS)
		{

			// Stream the specified audio into the speech engine
			//
			Audio_Stream(audio_file_path);

		}


		// Close the speech port
		//
		int ErrorCode = LV_SRE_DestroyClient(hPort);

		if(ErrorCode != LV_SUCCESS)
			printf("Problem closing port: %s\n", LV_SRE_ReturnErrorString(ErrorCode));
	}

	printf("\n\nFinished - press any key to quit\n");

	_getche();
}


